from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^random/$', views.random_kitten_view, name='random-kitten'),
    url(r'^up/(?P<id>[0-9]*)/$', views.kitten_up, name='kitten-up'),
    url(r'^down/(?P<id>[0-9]*)/$', views.kitten_down, name='kitten-down'),
    url(r'^top/', views.top_kittens, name='top-kittens'),
]



from django_dynamic_fixture import G
from django.test import TestCase
from mock import patch, MagicMock

from django_kittens.models import Kitten, reddit_kitten


class GetRandomTest(TestCase):
    @patch('django_kittens.models.random.random')
    @patch('django_kittens.models.KittenManager._rand_inst')
    def test_calls_rand_inst(self, rand_inst_mock, random_mock):
        G(Kitten)
        random_mock.return_value = .00001
        Kitten.objects.get_random()
        self.assertTrue(rand_inst_mock.called)
        
    @patch('django_kittens.models.random.random')
    @patch('django_kittens.models.KittenManager.create_new')
    def test_calls_create_new(self, create_new_mock, random_mock):
        G(Kitten)
        random_mock.return_value = .9
        Kitten.objects.get_random()
        self.assertTrue(create_new_mock.called)


class RandInstTest(TestCase):
    def test_returns_inst(self):
        k = G(Kitten)
        res = Kitten.objects._rand_inst()
        self.assertEqual(res, k)


class CreateNewTest(TestCase):
    def setUp(self):
        self.kit = MagicMock()
        self.kit.url = 'abc.com'
        self.kit.thumbnail = 'cute.jpg'
        self.kit.title = 'My cute cat'

    @patch('django_kittens.models.reddit_kitten')
    def test_saves_instance(self, reddit_kitten_mock):
        reddit_kitten_mock.return_value = self.kit
        Kitten.objects.create_new()
        kitten_count = Kitten.objects.count()
        expected = 1
        self.assertEqual(kitten_count, expected)

    @patch('django_kittens.models.reddit_kitten')
    def test_returns_instance(self, reddit_kitten_mock):
        reddit_kitten_mock.return_value = self.kit
        kit = Kitten.objects.create_new()
        self.assertTrue(kit)



from django.shortcuts import render_to_response

from .models import Kitten
from . import constants


def random_kitten_view(request, *args, **context):
    kitten = Kitten.objects.get_random()
    context['kitten'] = kitten
    context['kittens_home'] = constants.KITTEN_HOMEPAGE
    return render_to_response("kittens/random.html", context)


def kitten_up(request, *args, **context):
    kitten = Kitten.objects.get(id=context['id'])
    kitten.votes_up += 1
    kitten.save()
    context['kitten'] = kitten
    context['kittens_home'] = constants.KITTEN_HOMEPAGE
    return render_to_response("kittens/kitten_up.html", context)


def kitten_down(request, *args, **context):
    kitten = Kitten.objects.get(id=context['id'])
    kitten.votes_down += 1
    kitten.save()
    context['kitten'] = kitten
    context['kittens_home'] = constants.KITTEN_HOMEPAGE
    return render_to_response("kittens/kitten_down.html", context)


def top_kittens(request, *args, **context):
    top = Kitten.objects.extra(
        select={'net_score': 'votes_up - votes_down'}
    ).extra(where=['net_score > 0'], order_by=['-net_score'])[0:10]
    context['kittens'] = top
    context['kittens_home'] = constants.KITTEN_HOMEPAGE
    return render_to_response("kittens/top.html", context)



from django.conf import settings

KITTEN_HOMEPAGE = getattr(settings, 'KITTEN_HOMEPAGE', '/')
KITTEN_FRESHNESS = getattr(settings, 'KITTEN_FRESHNESS', 100)
REDDIT_USER_AGENT = getattr(settings, 'REDDIT_USER_AGENT', 'django-kittens')



import re
from setuptools import setup, find_packages


def get_version():
    """
    Extracts the version number from the version.py file.
    """
    VERSION_FILE = 'django_kittens/version.py'
    mo = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', open(VERSION_FILE, 'rt').read(), re.M)
    if mo:
        return mo.group(1)
    else:
        raise RuntimeError('Unable to find version string in {0}.'.format(VERSION_FILE))


setup(
    name='django-kittens',
    version=get_version(),
    description='Add kittens and kitten voting to your django project!',
    long_description=open('README.rst').read(),
    url='https://github.com/Wilduck/django-kittens',
    author='Erik Swanson',
    author_email='TheErikSwanson@gmail.com',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Framework :: Django :: 1.8',
    ],
    license='MIT',
    package_data={
        'django_kittens': [
            'templates/kittens/*.html',
            'static/kittens/*.css',
        ]
    },
    install_requires=[
        'django>=1.8',
        'praw',
    ],
    tests_require=[
        'django-nose',
        'mock>=1.0.1',
        'django-dynamic-fixture',
    ],
    setup_requires=[
        'nose>=1.0',
    ],
    test_suite="run_tests.run_tests",
    zip_safe=False,
)



import random

from django.db import models
import praw

from . import constants


class KittenManager(models.Manager):
    def get_random(self):
        """Get a kitten, either from the db, or a new one.
        """
        num_kittens = self.count()
        new_cutoff = (num_kittens / (num_kittens + constants.KITTEN_FRESHNESS))
        if random.random() < new_cutoff:
            return self._rand_inst()
        else:
            return self.create_new()

    def _rand_inst(self):
        ids = self.values_list('id', flat=True)
        rand_id = random.choice(ids)
        return self.get(id=rand_id)

    def create_new(self):
        """Create a new kitten by querying /r/awww.
        """
        kitten = reddit_kitten()
        if self.filter(url=kitten.url).exists():
            return self.get(url=kitten.url)
        else:
            kitten_obj = self.create(
                url=kitten.url,
                thumbnail=kitten.thumbnail,
                title=kitten.title,
            )
            return kitten_obj


class Kitten(models.Model):
    url = models.CharField(max_length=200, unique=True)
    thumbnail = models.CharField(max_length=200)
    title = models.CharField(max_length=200, unique=True)
    votes_up = models.PositiveIntegerField(default=0)
    votes_down = models.PositiveIntegerField(default=0)
    added = models.DateTimeField(auto_now_add=True)

    objects = KittenManager()


def reddit_kitten():
    reddit_api = praw.Reddit(user_agent=constants.REDDIT_USER_AGENT)
    aww_subreddit = reddit_api.get_subreddit("aww")
    kitten_results = aww_subreddit.search("kitten", sort="new", limit=100)
    kittens = [k for k in kitten_results if k.thumbnail != 'self']
    return random.choice(kittens)
